/*
 * Test_M644_FT810.c
 *
 * Created: 18.05.2016 18:12:19
 * Author : Rudolph Riedel
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#define F_CPU 16000000UL

#include <util/delay.h>

#include "FT800.h"
#include "FT800_commands.h"
#include "pic_data.h"


volatile uint8_t roundtrip=0;

// called every 250s
ISR (TIMER1_COMPA_vect)
{
	roundtrip++;
	roundtrip &= 0x03; // limit to 4 time-slots
}


void init_timer1(void)
{
	// Timer1 - 250s @ 2 Mhz
	TCCR1B = 0x00; // stop
	TCCR1A = 0x00;
	TCCR1C = 0x00;
	OCR1A = 499; // TOP - timer zhlt bis TOP und wirft nen IRQ
	TIMSK1 |= (1<<OCIE1A);
	TCCR1B = (1<<WGM12) | (1<<CS11); // mode = CTC, prescaler = 8 -> 2 MHz
}


void init_ports(void)
{
	DDRA = 0x00;  // all Pins to Input
	PORTA = 0xff; // Pullups on

	DDRB = (1<<PB2) | (1<<PB3) | (1<<PB4) | (1<<PB5) | (1<<PB7);  // SS (not used), SCK, MOSI, PB3=CS, PB2=PowerDown to Output, others to Input
	PORTB = (1<<PB4) | (1<PB3); // SS (not used) to High, CS to High, PowerDown to Low

	DDRC = 0x00;  // all pins to Input
	PORTC = 0xff; // Pullups on

	DDRD = 0x00; // all Pins to Input
	PORTD = 0x9f; // Pullups on
}


void init_spi(void)
{
	SPCR = (1<<SPE) | (1<<MSTR);	// SPI on, MSB first, Mode 0, Master, Fosc/4 -> 4MHz @ 16MHz CPU
	SPSR = (1<<SPI2X); // Fosc/2 -> 8 MHz @ 16MHz CPU
}


void ft800_loop()
{
	uint32_t calc;
	static uint8_t tag=0, pic=0, pic2=0;
	static uint16_t button = 0;
	static uint16_t opt_keys=0;
	static uint16_t dot_rad = 0;
	static uint8_t delay;
	static uint16_t progress = 0;
	static uint16_t toggle_state = 0;
	static uint8_t toggle_lock = 0;
	static int32_t rotate = 0;
	static int16_t x2 = 215;
	static uint8_t upx = 0;
	static uint16_t slider_val = 2048;
	static uint8_t track = 0;
	static uint32_t tracker_val = 0;
	static uint16_t scroller_val = 1024;
	static int16_t xpos_rel = 0;

	switch(delay)
	{
		case 0:
			calc = ft800_get_touch_tag();
			tag = calc;
			button = 0;
			pic = 0;
			pic2 = 0;
			opt_keys = 0;
			dot_rad = 0;
				
			switch(tag)
			{
				case 0:
					toggle_lock = 0;
					if(track != 0)
					{
						ft800_cmd_track(0, 0, 0, 0, 0); // stop tracking
						ft800_cmd_execute();
						track = 0;
					}
					break;

				case 1:
					button = FT_OPT_FLAT;
					break;

				case 4:
					pic = 1;
					break;

				case 5:
					pic2 = 1;
					break;

				case 6:
					break;

				case 8:
					if(toggle_lock == 0)
					{
						toggle_lock = 42;
						if(toggle_state == 0)
						{
							toggle_state = 0xffff;
						}
						else
						{
							toggle_state = 0;
						}
					}
					break;

				case 7:		// scroller
					if(track == 0)
					{
						ft800_cmd_track(5,190,85,10,7); // start tracking
						ft800_cmd_execute();
						track = 42;
					}
					else
					{
						tracker_val = ft800_memRead32(REG_TRACKER);
						if((tracker_val & 0xff) == 7)
						{
							tracker_val = (tracker_val >> 20);
							scroller_val = tracker_val;
						}
					}
					break;

				case 10:	// slider
					if(track == 0)
					{
						ft800_cmd_track(5,220,150,10,10); // start tracking
						ft800_cmd_execute();
						track = 42;
					}
					else
					{
						tracker_val = ft800_memRead32(REG_TRACKER);
						if((tracker_val & 0xff) == 10)
						{
							tracker_val = (tracker_val >> 20);
							slider_val = tracker_val;
							xpos_rel = tracker_val;
							xpos_rel /= 32;
							xpos_rel -= 64;
						}
					}
					break;

				case 49:
					opt_keys = '1';
					dot_rad = 10;
					break;

				case 50:
					opt_keys = '2';
					dot_rad = 15;
					break;

				case 51:
					opt_keys = '3';
					dot_rad = 20;
					break;

				case 52:
					opt_keys = '4';
					dot_rad = 20;
					break;

				case 53:
					opt_keys = '5';
					dot_rad = 25;
					break;
			}


		case 4:
			ft800_cmd_dl(CMD_DLSTART); // Start the display list
			ft800_cmd_dl(DL_CLEAR_RGB | BLACK); // Set the default clear color to black
			ft800_cmd_dl(DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG); // Clear the screen - this and the previous prevent artifacts between lists, Attributes are the color, stencil and tag buffers
			ft800_cmd_dl(TAG(0));
			break;

		case 5:
			ft800_cmd_text(xpos_rel+10, 20, 29, 0, "DING!");
			break;

		case 6:
			ft800_cmd_text(xpos_rel+20, 400, 28, 0, "the quick brown fox jumps over the lazy dog");
			break;

		case 7:
			ft800_cmd_keys(xpos_rel+90, 25, 100, 20, 26, opt_keys, "12345");
			ft800_cmd_dl(DL_COLOR_RGB | ORANGE);
			ft800_cmd_dl(TAG(9));
			ft800_cmd_line(xpos_rel+215, 55, xpos_rel+310, 55, 1);
			ft800_cmd_line(xpos_rel+215, 55, xpos_rel+215, 100, 1);
			ft800_cmd_line(xpos_rel+215, 55, xpos_rel+x2, 100, 1);
			break;

		case 8:
			ft800_cmd_dl(TAG(1));
			ft800_cmd_dl(DL_COLOR_RGB | WHITE);
			ft800_cmd_fgcolor(0x101010);
			ft800_cmd_gradcolor(RED);
			ft800_cmd_button(xpos_rel+220, 25, 80, 20, 26, button, "Cat");
			ft800_cmd_fgcolor(0x003870); // back to default, light-blue
			break;

		case 9:
			ft800_cmd_dl(DL_COLOR_RGB | RED);
			ft800_cmd_dl(TAG(2));
			ft800_cmd_point(xpos_rel+260, 70, 5);

			ft800_cmd_dl(DL_COLOR_RGB | BLUE);
			ft800_cmd_dl(TAG(3));
			ft800_cmd_point(xpos_rel+280, 85, 8);

			if(dot_rad != 0)
			{
				ft800_cmd_dl(DL_COLOR_RGB | GREEN);
				ft800_cmd_point(xpos_rel+250, 100, dot_rad);
			}
			break;

		case 10:
			ft800_cmd_dl(DL_COLOR_RGB | WHITE);

			if(toggle_state == 0)
			{
				ft800_cmd_dl(TAG(4));
				ft800_cmd_dl(DL_BEGIN | FT_BITMAPS);
				ft800_cmd_dl(BITMAP_SOURCE(18432*pic));
				ft800_cmd_dl(BITMAP_LAYOUT(FT_RGB565,96*2,96)); // jpg3
				ft800_cmd_dl(BITMAP_SIZE(FT_NEAREST,FT_BORDER,FT_BORDER,96,96)); // jpg3
				ft800_cmd_dl(VERTEX2F((xpos_rel+10)*16,50*16));
//				ft800_cmd_dl(VERTEX2II(xpos_rel+10,50,0,pic));
				ft800_cmd_dl(DL_END);
			}
			else
			{
				ft800_cmd_dl(TAG(0));
				ft800_cmd_dial(xpos_rel+40, 98, 32, 0, 0x8000);
			}
			ft800_cmd_dl(TAG(0));
			ft800_cmd_dl(DL_COLOR_RGB | PURPLE);
			ft800_cmd_rect(xpos_rel+120, 190, xpos_rel+180, 210, 3);
			ft800_cmd_dl(DL_COLOR_RGB | WHITE);
			break;

		case 11:
			if(toggle_state == 0)
			{
				ft800_cmd_dl(TAG(5));
				ft800_cmd_dl(DL_BEGIN | FT_BITMAPS);
				ft800_cmd_dl(BITMAP_SOURCE(40000));
				ft800_cmd_dl(BITMAP_LAYOUT(FT_RGB565,80*2,105));
				ft800_cmd_dl(BITMAP_SIZE(FT_NEAREST,FT_BORDER,FT_BORDER,80,105));
				ft800_cmd_dl(VERTEX2II(xpos_rel+110,50,0,pic2));
				ft800_cmd_dl(DL_END);
			}
			else
			{
				ft800_cmd_dl(TAG(0));
				ft800_cmd_gauge(xpos_rel+120, 98, 40, 0, 5, 4, slider_val,4095);
			}
			break;

		case 12:
			ft800_cmd_dl(TAG(0));
			ft800_cmd_text(xpos_rel+0, 150, 26, 0, "TAG:");
			ft800_cmd_number(xpos_rel+35, 150, 26, 0, tag);
			break;

		case 13:
//			ft800_cmd_dl(DL_COLOR_RGB | GREEN);
			ft800_cmd_progress(xpos_rel+5,170,85,10,0,progress,500);

			ft800_cmd_dl(TAG(7));
			ft800_cmd_scrollbar(xpos_rel+5,190,85,10,0,scroller_val,1024,4096);

			ft800_cmd_dl(TAG(10));
			ft800_cmd_slider(5,220,150,10,0,slider_val,4095);
			break;

		case 14:
			ft800_cmd_dl(TAG(8));
			ft800_cmd_toggle(xpos_rel+120, 165, 33, 26, 0, toggle_state, "aus" "\xff" "an");
			
//			ft800_cmd_dl(TAG(0));
//			ft800_cmd_number(5, 225, 26, 0, trackerval);
			break;

		case 15:
			ft800_cmd_dl(TAG(0));
			ft800_cmd_dl(BITMAP_SOURCE(74000));
			ft800_cmd_dl(BITMAP_LAYOUT(FT_RGB565,64*2,64));
			ft800_cmd_dl(BITMAP_SIZE(FT_NEAREST,FT_BORDER,FT_BORDER,64,64));

			if(toggle_state == 0)
			{
				rotate = 0;
			}
			else
			{
				ft800_cmd_loadidentity();
				ft800_cmd_translate(65536 * 32, 65536 * 32);
				ft800_cmd_rotate(rotate);
				ft800_cmd_translate(65536 * -32, 65536 * -32);
				ft800_cmd_setmatrix();
				rotate += 256;
			}

			ft800_cmd_dl(DL_BEGIN | FT_BITMAPS);
			ft800_cmd_dl(VERTEX2II(xpos_rel+192,112,0,4));
			ft800_cmd_dl(DL_END);
			break;

		case 16:
			ft800_cmd_dl(TAG(0));

			ft800_cmd_dl(BITMAP_SOURCE(74000));
			ft800_cmd_dl(BITMAP_LAYOUT(FT_RGB565,64*2,64));
			ft800_cmd_dl(BITMAP_SIZE(FT_NEAREST,FT_BORDER,FT_BORDER,64,64));

			ft800_cmd_dl(DL_BEGIN | FT_BITMAPS);
			ft800_cmd_dl(VERTEX2II(32+64*0,300,0,0));
			ft800_cmd_dl(VERTEX2II(32+64*1,300,0,1));
			ft800_cmd_dl(VERTEX2II(32+64*2,300,0,2));
			ft800_cmd_dl(VERTEX2II(32+64*3,300,0,3));
			ft800_cmd_dl(VERTEX2II(32+64*4,300,0,4));
			ft800_cmd_dl(DL_END);

			ft800_cmd_dl(BITMAP_SOURCE(74000));
			ft800_cmd_dl(BITMAP_LAYOUT(FT_RGB565,64*2,64));
			ft800_cmd_dl(BITMAP_SIZE(FT_NEAREST,FT_BORDER,FT_BORDER,64,64));

			if(toggle_state != 0) // x-flip an icon
			{
				ft800_cmd_loadidentity();
				ft800_cmd_translate(2097152, 2097152);
				ft800_cmd_scale(-65536, 65536);
				ft800_cmd_translate(-2097152, -2097152);
				ft800_cmd_setmatrix();
			}

			ft800_cmd_dl(DL_BEGIN | FT_BITMAPS);
			ft800_cmd_dl(VERTEX2II(xpos_rel+256,112,0,0));
			ft800_cmd_dl(DL_END);
			break;

		case 24:
			ft800_cmd_dl(DL_DISPLAY);	// Instruct the graphics processor to show the list
			ft800_cmd_dl(CMD_SWAP); // Make this list active
			ft800_cmd_execute();
			break;
	}

	delay++;
	if(delay > 24)
	{
		delay = 0;

		progress++;
		if(progress > 500)
		{
			progress = 0;
		}

		if(upx == 0)
		{
			x2+=1;
			if(x2 > 310)
			{
				x2 = 310;
				upx = 42;
			}
		}
		else
		{
			x2-=1;
			if(x2 < 215)
			{
				x2 = 215;
				upx = 0;
			}
		}
	}
}



int main(void)
{
	uint8_t triplock;

	cli();
	init_ports();
	init_timer1();
	init_spi();
	sei();

	ft800_init();
	ft800_memWrite8(REG_ROTATE, 1);	// rotate display by 180

	while(ft800_busy() == 1);
	ft800_get_cmdoffset();

	ft800_cmd_memset(0, 0xff,1L*96*96*2);	// Clear the memory at location 0 - any previous junk or bitmap data
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(0, FT_OPT_NODL, jpg3, 3708);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(18432, FT_OPT_NODL, jpg4, 2640);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(40000, FT_OPT_NODL, leopard, 3741);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(56800, FT_OPT_NODL, schneehase, 1833);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(74000 + (8192 * 0), FT_OPT_NODL, blitz, 1638);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(74000 + (8192 * 1), FT_OPT_NODL, figure, 2314);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(74000 + (8192 * 2), FT_OPT_NODL, frog, 1639);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(74000 + (8192 * 3), FT_OPT_NODL, map1, 2476);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	ft800_cmd_loadimage(106768, FT_OPT_NODL, spiral, 2618);
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	// send pre-recorded touch calibration values
	ft800_memWrite32(REG_TOUCH_TRANSFORM_A, 0x000001ac);
	ft800_memWrite32(REG_TOUCH_TRANSFORM_B, 0xffff29f5);
	ft800_memWrite32(REG_TOUCH_TRANSFORM_C, 0x033bb016);
	ft800_memWrite32(REG_TOUCH_TRANSFORM_D, 0xffff7006);
	ft800_memWrite32(REG_TOUCH_TRANSFORM_E, 0xfffffd1e);
	ft800_memWrite32(REG_TOUCH_TRANSFORM_F, 0x02201385);

/*
	// calibrate touch and displays values to screen
	ft800_cmd_dl(CMD_DLSTART); // Start the display list
	ft800_cmd_dl(DL_CLEAR_RGB | BLACK); // Set the default clear color to black
	ft800_cmd_dl(DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG); // Clear the screen - this and the previous prevent artifacts between lists, Attributes are the color, stencil and tag buffers
	ft800_cmd_text((FT_HSIZE/2), (FT_VSIZE/2), 27, FT_OPT_CENTER, "Please tap on the dot.");
	ft800_cmd_calibrate();
	ft800_cmd_dl(DL_DISPLAY);	// Instruct the graphics processor to show the list
	ft800_cmd_dl(CMD_SWAP); // Make this list active
	ft800_cmd_execute();

	while(ft800_busy() == 1);

	uint32_t touch_a, touch_b, touch_c, touch_d, touch_e, touch_f;

	touch_a = ft800_memRead32(REG_TOUCH_TRANSFORM_A);
	touch_b = ft800_memRead32(REG_TOUCH_TRANSFORM_B);
	touch_c = ft800_memRead32(REG_TOUCH_TRANSFORM_C);
	touch_d = ft800_memRead32(REG_TOUCH_TRANSFORM_D);
	touch_e = ft800_memRead32(REG_TOUCH_TRANSFORM_E);
	touch_f = ft800_memRead32(REG_TOUCH_TRANSFORM_F);

	ft800_cmd_dl(CMD_DLSTART); // Start the display list
	ft800_cmd_dl(DL_CLEAR_RGB | BLACK); // Set the default clear color to black
	ft800_cmd_dl(DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG); // Clear the screen - this and the previous prevent artifacts between lists, Attributes are the color, stencil and tag buffers
	ft800_cmd_dl(TAG(0));

	ft800_cmd_text(5, 30, 28, 0, "TOUCH_TRANSFORM_A:");
	ft800_cmd_text(5, 50, 28, 0, "TOUCH_TRANSFORM_B:");
	ft800_cmd_text(5, 70, 28, 0, "TOUCH_TRANSFORM_C:");
	ft800_cmd_text(5, 90, 28, 0, "TOUCH_TRANSFORM_D:");
	ft800_cmd_text(5, 110, 28, 0, "TOUCH_TRANSFORM_E:");
	ft800_cmd_text(5, 130, 28, 0, "TOUCH_TRANSFORM_F:");

	ft800_cmd_setbase(16L);
	ft800_cmd_number(250, 30, 28, 0, touch_a);
	ft800_cmd_number(250, 50, 28, 0, touch_b);
	ft800_cmd_number(250, 70, 28, 0, touch_c);
	ft800_cmd_number(250, 90, 28, 0, touch_d);
	ft800_cmd_number(250, 110, 28, 0, touch_e);
	ft800_cmd_number(250, 130, 28, 0, touch_f);

	ft800_cmd_dl(DL_DISPLAY);	// Instruct the graphics processor to show the list
	ft800_cmd_dl(CMD_SWAP); // Make this list active
	ft800_cmd_execute();

	while(1); // just stop here
*/

	set_sleep_mode(SLEEP_MODE_IDLE);
	
	while(1)
	{
		triplock = roundtrip;

		if(roundtrip == 0)
		{
			ft800_loop();
		}

		if(roundtrip == 1)
		{
		}

		if(roundtrip == 2)
		{
		}

		if(roundtrip == 3)
		{
		}

		while(triplock == roundtrip)
		{
			cli();
			sleep_enable();
			sei();
			sleep_cpu();
			sleep_disable();
		}
	} // while
}
